/*
 *
 * (c) by Alexander Neumann <alexander@bumpern.de>
 * Copyright(c) 2008 Christian Dietrich <stettberger@dokucode.de>
 * Copyright(c) 2008 Stefan Siegl <stesie@brokenpipe.de>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License (either version 2 or
 * version 3) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For more information on the GPL, please go to:
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <util/crc16.h>
#include <avr/pgmspace.h>

#include "config.h"
#include "core/eeprom.h"



size_t strlcpy_E(void *dst, const void *src, size_t size) {

 uint8_t c;
 size_t len=0;

 while((c=eeprom_read_byte((uint8_t*)src++))) {
  if (size == len) break;
  *(uint8_t*)dst++=c;
  len++;
 }
 *(uint8_t*)dst=0;
 return len;
}



size_t eeprom_write_str(const void *src, void *dst, size_t max_n) {

 size_t len = (strlen((const char*)src) >= max_n)?max_n:strlen((const char*)src)+1;

 eeprom_update_block(src, dst, len);
 return len;
}



uint16_t eeprom_calc_crc(const void *ptr, size_t size) {

 uint16_t crc = 0xffff;

 while(size--)
   crc = _crc16_update(crc, eeprom_read_byte((uint8_t*)ptr++));
 return crc;
} 



uint16_t eeprom_check_crc(const void *ptr, size_t size) {

 uint16_t crc;

 crc = eeprom_calc_crc(ptr, size);
 crc = _crc16_update(crc, eeprom_read_byte((uint8_t*)(ptr+size)));
 crc = _crc16_update(crc, eeprom_read_byte((uint8_t*)(ptr+size+1)));
 return crc;
}



uint16_t eeprom_update_crc(const void *ptr, size_t size) {

 uint16_t crc;

 crc = eeprom_calc_crc(ptr, size);
 eeprom_write_byte((uint8_t*)(ptr+size), crc & 255);
 eeprom_write_byte((uint8_t*)(ptr+size+1), crc >> 8);
 return crc;
}



#ifdef CRC_SUPPORT
uint8_t
crc_checksum (void *data, uint8_t length)
{

  uint8_t crc = 0;
  uint8_t *p = (uint8_t *) data;

  for (uint8_t i = 0; i < length; i++)
    {
      crc = _crc_ibutton_update (crc, *p);
      p++;
    }

  return crc;

}
#endif /* CRC_SUPPORT */



// Dummy-shit

#ifdef EEPROM_SUPPORT

void
eeprom_write_block_hack (void *dst, const void *src, size_t n)
{
}


uint8_t
eeprom_get_chksum (void)
{
  return 0;
}



void
eeprom_init (void)
{
}


#endif /* EEPROM_SUPPORT */
