/*
 * Copyright (c) 2010 by Christian Dietrich <stettberger@dokucode.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For more information on the GPL, please go to:
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <stdint.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <avr/eeprom.h>
#include "core/global.h"
#include "core/eeprom.h"
#include "protocols/ecmd/ecmd-base.h"
#include "core/util/string_parsing.h"

#define ROWS 16
#define LINES 16
#define OFFSET 6


int8_t hexdigit(char *cmd)
{
 if (*cmd >= '0' && *cmd <= '9') return *cmd - '0';
 if (*cmd >= 'a' && *cmd <= 'f') return *cmd - 'a' + 10;
 if (*cmd >= 'A' && *cmd <= 'F') return *cmd - 'A' + 10;
 return -1;
}

/* Read hexword at cmd, strips all spaces before and returns consumed
   characters or 0 on error */
uint8_t
next_hexword(char *cmd, uint16_t *value, uint8_t maxdigits)
{
    char *old_cmd = cmd;
    while (*cmd == ' ') cmd++; /* Strip the spaces before */
    
    *value = 0;
    int8_t tmp;

    while ((tmp = hexdigit(cmd)) >= 0) {
       (*value) = ((*value) << 4) + tmp;
       cmd++;
       if (--maxdigits == 0) break; 
    }
    return cmd - old_cmd;
}


int16_t parse_cmd_eedump(char *cmd, char *output, uint16_t len)
{
 static uint16_t address;
 static uint8_t line;
 uint8_t i, tmp;


  if (cmd[0] != 0x05) {
    next_hexword(cmd, &address, 4);
    line = LINES;
    cmd[0] = 0x05;
  }

 sprintf_P(output, PSTR("%04X: "), address);
 for (i = 0; i < ROWS; i++) {
    tmp = eeprom_read_byte((uint8_t*) address++);
    sprintf_P(output + OFFSET + 3*i, PSTR("%02X "), tmp);
    //sprintf_P(output + 5 + 3*16 + 1 + i, PSTR("%c"), isprint(tmp)?tmp:'.');
    output[OFFSET + 3*ROWS + 1 + i] = isprint(tmp)?tmp:'.';
 }
 output[OFFSET + 3*ROWS] = ' ';
 output[OFFSET + 4*ROWS + 1] = 0;
 if (--line)
   return ECMD_AGAIN(OFFSET + 4*ROWS + 1);
 else
   return ECMD_FINAL(OFFSET + 4*ROWS + 1);
}


int16_t parse_cmd_dump(char *cmd, char *output, uint16_t len)
{
 static uint8_t *address;
 static uint8_t line;
 uint8_t i, tmp;


  if (cmd[0] != 0x05) {
    next_hexword(cmd, (uint16_t*) &address, 4);
    line = LINES;
    cmd[0] = 0x05;
  }

 sprintf_P(output, PSTR("%04X: "), address);
 for (i = 0; i < ROWS; i++) {
    tmp = *address++;
    sprintf_P(output + OFFSET + 3*i, PSTR("%02X "), tmp);
    //sprintf_P(output + 5 + 3*16 + 1 + i, PSTR("%c"), isprint(tmp)?tmp:'.');
    output[OFFSET + 3*ROWS + 1 + i] = isprint(tmp)?tmp:'.';
 }
 output[OFFSET + 3*ROWS] = ' ';
 output[OFFSET + 4*ROWS + 1] = 0;
 if (--line)
   return ECMD_AGAIN(OFFSET + 4*ROWS + 1);
 else
   return ECMD_FINAL(OFFSET + 4*ROWS + 1);
}

#if 0
int16_t parse_cmd_eer(char *cmd, char *output, uint16_t len)
{
    uint16_t addr_offset, length;

    uint8_t p = next_uint16(cmd, &addr_offset);
    if (p == 0)
        return ECMD_ERR_PARSE_ERROR;

    p = next_uint16(cmd + p, &length);
    if (p == 0) 
        length = 1;

    if (length * 2 >= len)
        length = len / 2;

    if (0xffff - eeprom_start - length < addr_offset)
        return ECMD_ERR_PARSE_ERROR;
    
    uint16_t ptr = eeprom_start + addr_offset;
    for (uint16_t i = 0; i < length; i ++)
      sprintf_P (output + (i << 1), PSTR("%02x"), 
                 eeprom_read_byte((uint8_t *) (ptr ++)));
    
    return ECMD_FINAL(length * 2);
}



int16_t parse_cmd_eew(char *cmd, char *output, uint16_t len)
{
    uint16_t addr_offset;

    uint8_t p = next_uint16(cmd, &addr_offset);
    if (p == 0)
        return ECMD_ERR_PARSE_ERROR;

    cmd += p;
    
    uint16_t ptr = eeprom_start + addr_offset;
    char *cmd_end = cmd + strlen(cmd);

    uint8_t i = 0;
    while (cmd < cmd_end) {
        if (ptr < eeprom_start) 
            return ECMD_ERR_PARSE_ERROR;

        /* Read the next hex byte */
        uint8_t value;
        if (!(p = next_hexbyte(cmd, &value)))
            return sprintf(output, "%d hexbyte '%x'", i, value);
        cmd += p;

        eeprom_write_byte((uint8_t *) (ptr ++), value);
        i++;
    }
    
    return ECMD_FINAL_OK;
}
#endif


/*
  -- Ethersex META --
  block(Reading and Writing EEPROM Space on Device)
  ecmd_feature(eedump, "eedump",eedump <ADDR>, Dump 256 bytes from address in eeprom.)
  ecmd_feature(dump, "dump",dump <ADDR>, Dump 256 bytes from address in sram.)
*/
