/*
 * Copyright (c) 2009 by Stefan Riepenhausen <rhn@gmx.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For more information on the GPL, please go to:
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <avr/pgmspace.h>
#include "config.h"
#include "protocols/ecmd/ecmd-base.h"
#include "core/eeprom.h"
#include "syspar.h"


static uint8_t check4par(char **cmdptr)
{
 char *ptr = *cmdptr;

 while (*ptr == ' ')
     ptr++;
 if (*ptr == 0)
     return 0;
 if (*ptr == '"' && *(ptr+1) == '"' && *(ptr+2) == 0)
     **cmdptr = 0;
 else
     *cmdptr = ptr;
 return 1;
}


int16_t parse_cmd_dhcp(char *cmd, char *output, uint16_t len)
{
    uint8_t par = 0;

    while (*cmd == ' ')
        cmd++;
    if (*cmd != 0) {
        if (*cmd == 'Y' || *cmd == 'y')
           par = 255;
        eeprom_write_byte(&syspar_eeprom.syspar.netpar.use_dhcp, par);
        return ECMD_FINAL_OK;
    }
    else
    {
     par = eeprom_read_byte(&syspar_eeprom.syspar.netpar.use_dhcp);
     if (par)
       par = 'Y';
     else
       par = 'N';
     output[0] = par;
     return ECMD_FINAL(1);
    }
}


int16_t parse_cmd_hostname(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.hostname, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.hostname, MAX_STR_LEN ));
    }
}


int16_t parse_cmd_ntp_server1(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.ntp_server1, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.ntp_server1, MAX_STR_LEN ));
    }
}


int16_t parse_cmd_ntp_server2(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.ntp_server2, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.ntp_server2, MAX_STR_LEN ));
    }
}


int16_t parse_cmd_smtp_server(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.smtp_server, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.smtp_server, MAX_STR_LEN ));
    }
}


int16_t parse_cmd_smtp_login(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.smtp_login, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.smtp_login, MAX_STR_LEN ));
    }
}


int16_t parse_cmd_smtp_password(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.smtp_password, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.smtp_password, MAX_STR_LEN ));
    }
}


int16_t parse_cmd_smtp_addressee(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.smtp_addressee, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.smtp_addressee, MAX_STR_LEN ));
    }
}


int16_t parse_cmd_smtp_sender(char *cmd, char *output, uint16_t len)
{
    if (check4par(&cmd)) {
        eeprom_write_str(cmd, &syspar_eeprom.syspar.netpar.smtp_sender, MAX_STR_LEN);
        return ECMD_FINAL_OK;
    }
    else
    {
    return ECMD_FINAL(strlcpy_E(output, &syspar_eeprom.syspar.netpar.smtp_sender, MAX_STR_LEN ));
    }
}


/*
  -- Ethersex META --
  ecmd_feature(dhcp, "dhcp",,Display and set dhcp usage.)
  ecmd_feature(hostname, "hostname",,Display and set hostname.)
  ecmd_feature(ntp_server1, "ntp server1",,Display and set ntp server1.)
  ecmd_feature(ntp_server2, "ntp server2",,Display and set ntp server2.)
  ecmd_feature(smtp_server, "smtp server",,Display and set smtp server.)
  ecmd_feature(smtp_login, "smtp login",,Display and set smtp login.)
  ecmd_feature(smtp_password, "smtp password",,Display and set smtp password.)
  ecmd_feature(smtp_addressee, "smtp addressee",,Display and set smtp addressee.)
  ecmd_feature(smtp_sender, "smtp sender",,Display and set smtp sender.)
*/
