/*
 * Copyright (c) 2009 by Stefan Riepenhausen <rhn@gmx.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For more information on the GPL, please go to:
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <avr/pgmspace.h>
#include <avr/io.h>

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "main_control.h"
#include "key_deb.h"
#include "hardware/lcd/hd44780.h"
#include "protocols/ecmd/ecmd-base.h"
#include "protocols/uip/uip.h"
#include "protocols/dns/resolv.h"
#include "services/ntp/ntp.h"
#include "syspar.h"
#include "core/uart0.h"
#include "services/aquabox/debug_calling.h"


const char VERSIONS_F_STR[] PROGMEM = "MCU:  %S\nGCC:  %S\nLIBC: %S\n%S %S";
const char compile_date[] PROGMEM = __DATE__;
const char compile_time[] PROGMEM = __TIME__;
const char compiler_version[] PROGMEM = __VERSION__;
const char libc_version[] PROGMEM = __AVR_LIBC_VERSION_STRING__;
const char avr_type[] PROGMEM =
#if defined ( __AVR_ATmega16__ )
   "ATmega16";
#elif defined ( __AVR_ATmega32__ )
   "ATmega32";
#elif defined ( __AVR_ATmega644__ )
   "ATmega644";
#elif defined ( __AVR_ATmega644P__ )
   "ATmega644P";
#elif defined ( __AVR_ATmega1284P__ )
   "ATmega1284P";
#else
   "unknown";
#endif

volatile uint8_t EEMEM ta[256];

extern uint8_t mcusr_mirror;
extern uint16_t crc_mirror;
static uint8_t stop_it = 0;

uint8_t cmdbuf[40];


void debug_appcall(void)
{
 //struct ab_state *s=&(uip_udp_conn->appstate.ab);
 //struct ntp_packet *pkt = (void *) uip_appdata;

  static uint16_t packet_no = 0;
  uint16_t len;
  uint8_t cmdlen;  

  if (! uip_poll ())
    return;

  if (stop_it) {
    stop_it = 0;
    uip_udp_remove(uip_udp_conn);
    return;
  }

  if ((cmdlen = cmdbuf[0])) {
    len = sprintf_P((char*) uip_appdata, PSTR("%05u:"),++packet_no);
    for (uint8_t i = 0; i < cmdlen; i++)
       len += sprintf_P((char*) (uip_appdata + len), PSTR("%02X "), cmdbuf[i + 1]);
    len += sprintf_P((char*) (uip_appdata + len), PSTR("\n"));
    uip_udp_send(len);
    cmdbuf[0] = 0;
  }

#if 0
 switch (s->status) {

 case AB_IDLE:  len = sprintf_P((uint8_t*) uip_appdata, PSTR("\nAB-DEBUG:\n========\npacket_no=%u\n"),packet_no++);
                len += debug_out((uint8_t*) (uip_appdata + len));
                uip_udp_send(len);
                s->status = AB_SENT;
                break;

 case AB_ABORT:
 case AB_SENT:  uip_udp_remove(uip_udp_conn);
                s->status = AB_WAITING;
                break;
 }
#endif
}


void fifo_flush(void)
{
  while (ukbhit0())
    ugetchar0();
  return;
}


void got_desthost_address(char *name, uip_ipaddr_t *ipaddr)
{
 uip_udp_conn_t *debug_conn;

 if(ipaddr) {
   debug_conn = uip_udp_new(ipaddr, HTONS(DEBUG_PORT), debug_appcall);
   if(debug_conn) {
     uip_udp_bind(debug_conn, HTONS(6502)); // Immer den selben local Port benutzen
     cmdbuf[0] = 0;
     fifo_flush();
   }
 }
 return;
}


void start_debug(void)
{
 stop_it = 0;
 resolv_query("hell", got_desthost_address);
}


void stop_debug(void)
{
 stop_it = 1;
}


/*
  If enabled in menuconfig, this function is called during boot up of ethersex
*/
int16_t
main_control_init(void)
{
  printf_P(VERSIONS_F_STR, avr_type, compiler_version, libc_version, compile_date, compile_time);
  hd44780_goto(1,14);
  printf_P(PSTR("%04X"),eeprom_read_word((uint16_t*)&syspar_eeprom.crc16));
  hd44780_goto(2,14);
  printf_P(PSTR("%X %04X"),mcusr_mirror,crc_mirror);
  // enter your code here
  PIN_CLEAR(STATUSLED_SD_READER);

  init_uart0( (u16)(XTAL / BAUD / 16 - 0.5) );

  TCCR1A = (1<<COM1B1) | (1<<WGM11);
  TCCR1B = (1<<WGM13) | (1<<WGM12) | (1<<CS11);
  ICR1 = 46079;
  OCR1B = 3456;
  DDRD |= (1<<PD4);

  eeprom_check_crc(&ta, sizeof(ta));

  return 0;
}


void fifo_get_cmd_sync(void)
{
  static uint8_t state = 0;
  static uint8_t len, i;


  if (cmdbuf[0]) return;

  switch (state) {

  case 0:
       if (!ukbhit0()) return;
       if (ugetchar0() != 0xFE) {
          fifo_flush();
          return;
       }
       state = 1;

  case 1:
       if (!ukbhit0()) return;
       if (ugetchar0() != 0xED) {
          fifo_flush();
          state = 0;
          return;
       }
       state = 2;

  case 2:
       if (!ukbhit0()) return;
       if (ugetchar0() != 0xBA) {
          fifo_flush();
          state = 0;
          return;
       }
       state = 3;

  case 3:
       if (!ukbhit0()) return;
       if (ugetchar0() != 0xBE) {
          fifo_flush();
          state = 0;
          return;
       }
       state = 4;

  case 4:
       if (!ukbhit0()) return;
       if ((len = ugetchar0()) == 0) {
          fifo_flush();
          state = 0;
          return;
       }
       i = 0;
       state = 5;

  case 5:
       for (; i < len; i++) {
          if (!ukbhit0()) return;
          cmdbuf[i + 1] = ugetchar0();
       }
       cmdbuf[0] = len;
       state = 0;
  }
}

  
uint8_t fifo_sync_blocked(void)
{
  if (ugetchar0() != 0xFE) return 0;
  if (ugetchar0() != 0xED) return 0;
  if (ugetchar0() != 0xBA) return 0;
  if (ugetchar0() != 0xBE) return 0;
  return 1;
}


uint8_t fifo_get_cmd_blocked(void)
{
  if (ukbhit0()) {
    if (fifo_sync_blocked()) {
      uint8_t len = ugetchar0();
      for (uint8_t i = 0; i < len; i++) {
         cmdbuf[i] = ugetchar0();
      }
      return len;
    }
    else fifo_flush();
  }
  return 0;
}


int16_t
main_control(void)
{

  fifo_get_cmd_sync();
/*
  uint8_t cmdlen;  

  if ((cmdlen = fifo_get_cmd())) {
      hd44780_clear();
      hd44780_goto(0,0);
      for (uint8_t i = 0; i < cmdlen; i++)
        printf_P(PSTR("%02X "), cmdbuf[i]);
  }
*/
  return 0;
}


/*
  -- Ethersex META --
  header(services/aquabox/main_control.h)
  startup(main_control_init)
  timer(10, main_control())
*/

