/*
 * Copyright (c) 2012 by Volker Klautsch <derklautsch@web.de>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License (either version 2 or
 * version 3) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For more information on the GPL, please go to:
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "config.h"
#include "httpd.h"
#include "avr/pgmspace.h"
#include "hardware/storage/sd_reader/vfs_sd.h"
#include "hardware/storage/sd_reader/sd_raw.h"

#include "hardware/lcd/hd44780.h"



static long content_length;
static char upload_lock = 0;
static struct vfs_file_handle_t *ufh;

extern char uploadpath[];


char upload_locked(void)
{
 return upload_lock;
}


void cleanup_post(void)
{
 // ...
 upload_lock = 0;
}


void finish_file_upload(char *end_ptr)
{
  char *ptr = end_ptr;
  uint32_t len;

//  hd44780_clear();
//  hd44780_goto(0,0);
//  printf_P(PSTR("%02X %02X %02X %02X\n"), *ptr, *(ptr-1), *(ptr-2), *(ptr-3));

  ptr -= 4;
  while (ptr >= (char*)uip_appdata) {
    if (*ptr-- == 0x2d) break;
  }
  while (ptr >= (char*)uip_appdata) {
    if (*ptr-- != 0x2d) break;
  }
  while (ptr >= (char*)uip_appdata) {
    if (*ptr-- == 0x2d)
      if (*ptr == 0x2d) break;
  }
  while (ptr >= (char*)uip_appdata) {
    if (*ptr-- == 0x0a) break;
  }
  if (ptr >= (char*)uip_appdata && *ptr == 0x0d) {
    len = vfs_sd_size (ufh); 

//    printf_P(PSTR("Old-Len: %lu  \n"), len);

    len -= end_ptr - (ptr - 1);
    vfs_sd_truncate (ufh, len);
    httpd_handle_fu_ok ();
    STATE->handler = httpd_handle_fu_ok;

//    printf_P(PSTR("New-Len: %lu  "), len);

  }
  else {
    httpd_handle_500 ();
    STATE->handler = httpd_handle_500;
  }
  sd_raw_sync ();
  vfs_sd_close (ufh);
}


int init_file_upload(void)
{
  char *ptr = (char *) uip_appdata;
  char *filename, *start_payload_ptr, *start_content_ptr, *tmpptr1, *tmpptr2;
  
  ptr[uip_len] = 0;
  tmpptr1 = strstr_P(ptr, PSTR("Content-Length: "));
  if (tmpptr1 == NULL) return 1;
  tmpptr1+=16;
  tmpptr2 = strstr_P(tmpptr1, PSTR("\r\n\r\n"));
  if (tmpptr2 == NULL) return 2;
  *tmpptr2 = 0;
  start_content_ptr = tmpptr2 + 4;
  content_length = atol(tmpptr1);
  tmpptr1 = strstr_P(tmpptr2 + 1, PSTR("filename=\""));
  if (tmpptr1 == NULL) return 3;
  filename = tmpptr1 + 10;
  tmpptr2 = strstr_P(filename, PSTR("\"\r\n"));
  if (tmpptr2 == NULL) return 4;
  *tmpptr2 = 0;
  tmpptr1 = strstr_P(tmpptr2 + 1, PSTR("\r\n\r\n"));
  if (tmpptr1 == NULL) return 5;
  start_payload_ptr = tmpptr1 + 4;
  content_length -= (ptr + uip_len) - start_content_ptr;
  if (*filename == 0) return 6;

  int len = strlen(uploadpath);
  strcat (uploadpath, filename);
  ufh = vfs_sd_create (uploadpath);
  uploadpath[len] = 0;
  if (ufh == NULL) return 10;
  upload_lock = 1;
  vfs_sd_write (ufh, start_payload_ptr, (ptr + uip_len) - start_payload_ptr);
  if (content_length == 0L) {
    finish_file_upload(ptr + uip_len - 1);
    return 0;
  }
  vfs_sd_truncate (ufh, content_length);
  return 0;
}


void
httpd_handle_post (void)
{
  if(uip_newdata()) {
    if(upload_lock == 1) {
      upload_lock = 2;
      return;
    } 

    if (sd_active_partition == NULL) {
       httpd_cleanup ();
       uip_abort ();
       return;
    }

    vfs_sd_write (ufh, uip_appdata, uip_len);
    content_length -= uip_len;
    if (content_length == 0L) {
      if (vfs_sd_size (ufh) > 512L) {
        vfs_sd_fseek (ufh, -512L, FAT_SEEK_END);
      }
      else {
        vfs_sd_fseek (ufh, 0L, FAT_SEEK_SET);
      }
      content_length = vfs_sd_read (ufh, uip_appdata, 512L) - 1;
      finish_file_upload (uip_appdata + (uint16_t)content_length);
    }
  }
}

