/*
 * Copyright (c) 2009 by Stefan Siegl <stesie@brokenpipe.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For more information on the GPL, please go to:
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "config.h"
#include "httpd.h"
#include "httpd_state.h"
#include <stdio.h>
#include "hardware/storage/sd_reader/fat.h"

#include "hardware/lcd/hd44780.h"


const char PROGMEM httpd_sd_dir_header[] =
    "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">"
    "<head><title>Inhaltsverzeichnis</title></head>"
    "<html><body>"
    "<h2>Inhalt von %s</h2>"
    "<table><tr><th>Gr&ouml;&szlig;e</th><th width=\"35\"></th><th>Datum (UTC)</th><th width=\"35\"></th><th>Name</th></tr>\n";

const char PROGMEM httpd_sd_dir_parent[] =
    "<tr><td align=\"right\">-</td><td></td><td align=\"right\">-</td><td></td><td><a href='%stoc'>../</a></td></tr>";

const char PROGMEM httpd_sd_dir_footer[] =
    "</table><hr>"
    "<address>Ihr freundlicher Ethers&auml;x HTTP Service mit VFS und mod_m4 :-)</address>"
    "</body></html>";

const char PROGMEM httpd_sd_dir_entry[] =
    "<tr><td align=\"right\">%ld</td><td></td><td align=\"right\">%4d-%02d-%02d&nbsp;&nbsp;%02d:%02d</td><td></td><td><a href='%s'>%s</a></td></tr>\n";

const char PROGMEM httpd_sd_dir_entry_dir[] =
    "<tr><td align=\"right\">-</td><td></td><td align=\"right\">%4d-%02d-%02d&nbsp;&nbsp;%02d:%02d</td><td></td><td><a href='%stoc'>%s</a></td></tr>\n";

const char PROGMEM httpd_header_301_redirect[] =
    "HTTP/1.1 301 REDIRECT\n"
    "Location: %s/\n\n";


char uploadpath[SD_DIR_MAX_DIRNAME_LEN + 32];


static void
httpd_handle_sd_dir_send_header (void)
{

    strcpy (uploadpath, STATE->u.dir.dirname);
    if (strlen (uploadpath) > 1) {
       uploadpath[strlen (uploadpath) - 1] = 0;
    }
    PASTE_RESET ();
    PASTE_P (httpd_header_200);
    PASTE_P (httpd_header_ct_html);
    PASTE_PF (httpd_sd_dir_header, uploadpath);

    strcpy (uploadpath, STATE->u.dir.dirname);

    if (*STATE->u.dir.dirname != 0 && *(STATE->u.dir.dirname + 1) != 0) {
	/* We're listing a sub-directory, therefore let's insert a link
	   to the parent directory. */
	char *ptr = STATE->u.dir.dirname + strlen (STATE->u.dir.dirname) - 1;
	if (*ptr == '/') *ptr = 0;

	ptr = strrchr (STATE->u.dir.dirname, '/') + 1;

        *ptr = 0;		/* Chop of last directory's name. */

	PASTE_PF (httpd_sd_dir_parent, STATE->u.dir.dirname);
    }

    PASTE_SEND ();

}


void
httpd_handle_sd_dir (void)
{
    uint16_t year;
    uint8_t month, day, hour, min, sec;

    if (uip_acked ()) {
	if (!STATE->header_acked)
	    STATE->header_acked = 1;

	else if (STATE->eof) {
	    uip_close ();
	    return;
	}
	
	do
	    if (!fat_read_dir (STATE->u.dir.handle, &STATE->u.dir.entries)) {
		STATE->eof = 1;
		break;
	    }
	while (STATE->u.dir.entries.long_name[0] == '.');
	    
    }

    if (!STATE->header_acked)
	httpd_handle_sd_dir_send_header ();

    else if (STATE->eof) {
	/* Send footer. */
	PASTE_RESET ();
	PASTE_P (httpd_sd_dir_footer);
	PASTE_SEND ();
    }

    else {
	/* Send file entry. */

        fat_get_file_modification_date (&STATE->u.dir.entries, &year, &month, &day);
        fat_get_file_modification_time (&STATE->u.dir.entries, &hour, &min, &sec);

	PASTE_RESET ();

	if (STATE->u.dir.entries.attributes & FAT_ATTRIB_DIR) {
           char *ptr = STATE->u.dir.entries.long_name;
           ptr += strlen(ptr) - 1;
           if (*ptr != '/') {
              *(ptr + 1) = '/';
              *(ptr + 2) = 0;
            } 
	    PASTE_PF (httpd_sd_dir_entry_dir,
                      year, month, day, hour, min,
		      STATE->u.dir.entries.long_name,
		      STATE->u.dir.entries.long_name);
        }
	else
	    PASTE_PF (httpd_sd_dir_entry,
		      STATE->u.dir.entries.file_size,
                      year, month, day, hour, min,
		      STATE->u.dir.entries.long_name,
		      STATE->u.dir.entries.long_name);

	PASTE_SEND ();
    }
}


void
httpd_handle_sd_dir_redirect (void)
{
    if (uip_acked ())
	uip_close ();

    if (uip_poll ())
	return;

    PASTE_RESET ();
    PASTE_PF (httpd_header_301_redirect, STATE->u.dir.dirname);
    PASTE_SEND ();
}
