#ifndef __SYSPAR_H__
#define __SYSPAR_H__

#include <stdint.h>
#include "core/eeprom.h"
#include "protocols/uip/uip.h"


#define	IP_LEN		4
#define MAX_STR_LEN	32


typedef struct
{
  uint8_t month;
  uint8_t week;
  uint8_t dow;
  uint8_t hour;
} clock_dst_t;


typedef struct
{
  int32_t utctime;              /* Offset zu UTC in Sekunden */
  int32_t dsttime;              /* Offset zu Normalzeit in Sekunden */
  clock_dst_t dstbegin;
  clock_dst_t dstend;
} clock_timezone_t;


typedef struct {
uint8_t use_dhcp;
uint8_t use_dns;
uint8_t use_ntp;
uip_ipaddr_t static_host_ip;
uip_ipaddr_t static_router_ip;
uip_ipaddr_t static_dns_server_ip;
uip_ipaddr_t static_netmask;
uint8_t hostname[MAX_STR_LEN];
uint8_t ntp_server1[MAX_STR_LEN];
uint8_t ntp_server2[MAX_STR_LEN];
uint8_t smtp_server[MAX_STR_LEN];
uint8_t smtp_login[MAX_STR_LEN];
uint8_t smtp_password[MAX_STR_LEN];
uint8_t smtp_addressee[MAX_STR_LEN];
uint8_t smtp_sender[MAX_STR_LEN];
} netpar_t;


typedef struct {
uint8_t use_dst;
clock_timezone_t tz;
netpar_t netpar;
} syspar_t;


typedef struct {
union {
 struct {
  syspar_t syspar;
  uint16_t crc16;
 };
 uint8_t dummy[512];
};
} syspar_eeprom_t;


extern const char DEFAULT_HOSTNAME[];
extern const char DEFAULT_NTP_SERVER[];

extern syspar_eeprom_t syspar_eeprom;

int8_t init_syspar(void);

#endif /* __SYSPAR_H__ */
